/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucla.edu, imarkov@cs.ucla.edu
***  Affiliation:   UCLA, Computer Science Department,
***                 Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/




//  Created by Igor Markov, April 30, 1999
//  CHANGES

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "ABKCommon/sgi_hash_map.h"
#include "HGraphWDims/hgWDims.h"
#include "Placement/placeWOri.h"
#include "baseSmallPlPr.h"
#include "smallPlaceProb.h"

BitBoard SmallPlacementProblem::_seenNets(200);
BitBoard SmallPlacementProblem::_essentialNets(200);
BitBoard SmallPlacementProblem::_mobileCells(50);

SmallPlacementProblem::SmallPlacementProblem
	       (const HGraphWDimensions& hgwd, 
                const PlacementWOrient&  pl,
		const bit_vector&        placed,
	        const vector<unsigned>&  movables, 
	        const SmallPlacementRow& row,
		Verbosity                verb)

	:BaseSmallPlacementProblem(movables.size(),verb)
{
   _rows.push_back(row);

   unsigned k, numCells=movables.size();
   _mobileCells.reset(hgwd.getNumNodes());
   for(k=0; k!=numCells; k++) _mobileCells.setBit(movables[k]);

   _cellWidths.reserve(numCells);
   for(k=0; k!=numCells; k++)
   {
      _cellWidths.push_back(hgwd.getNodeWidth(movables[k]));
      abkassert(_cellWidths[k]!=-DBL_MAX,"uninitialized cell width");
   } 

   _seenNets.reset(hgwd.getNumEdges());
   _essentialNets.reset(hgwd.getNumEdges());

   for(k=0; k!=numCells; k++)
   {
      const HGFNode& node=hgwd.getNodeByIdx(movables[k]);
      for(itHGFEdgeLocal  e = node.edgesBegin(); e != node.edgesEnd(); e++)
      {
         const HGFEdge& edge=(**e);
         unsigned netIdx=edge.getIndex();
         if (!_seenNets.isBitSet(netIdx))
         {
            _seenNets.setBit(netIdx); 
            unsigned haveFixed=false;
            BBox box;
            unsigned nodeOffset=0;
            for(itHGFNodeLocal otherNode = edge.nodesBegin(); 
                otherNode!= edge.nodesEnd(); otherNode++, nodeOffset++)
            {
                unsigned nodeIndex=(*otherNode)->getIndex();
                if (_mobileCells.isBitSet(nodeIndex)) continue;

		if(placed[nodeIndex])
		{
                    Point pinOffset=hgwd.nodesOnEdgePinOffset
				(nodeOffset,netIdx,pl.getOrient(nodeIndex));
                    box+=(pl[nodeIndex]+pinOffset);
		}
		else
                    box+=pl[nodeIndex];

                haveFixed=true; 
            }
           
            if ( !haveFixed || box.xMin>row.xMin || box.xMax<row.xMax )
            {
               _netTerminalBBoxes.push_back(box);
               _essentialNets.setBit(netIdx);
            }
         }
      } 
   }

   const vector<unsigned>& essNetIds =_essentialNets.getIndicesOfSetBits();
   unsigned numNets=essNetIds.size();
   
   _netlist.setup(numCells,numNets);

   hash_map<unsigned, unsigned, hash<unsigned>, equal_to<unsigned> > 
                origToNewIdx;
   for(k=0; k!=numCells; k++) origToNewIdx[movables[k]]=k;


	//setup the pin offsets for the movable cells
   for(k=0; k!=numNets; k++)
   {
      unsigned netIndex=essNetIds[k];
      const HGFEdge& edge=hgwd.getEdgeByIdx(netIndex);
      unsigned nodeOffset=0;
      for(itHGFNodeLocal node = edge.nodesBegin(); 
                               node!= edge.nodesEnd(); node++, nodeOffset++)
      {
          unsigned nodeIndex=(*node)->getIndex();
          if (!_mobileCells.isBitSet(nodeIndex)) continue;
          Point pinOffset=hgwd.nodesOnEdgePinOffset
				(nodeOffset,netIndex,pl.getOrient(nodeIndex));

          _netlist.setPinOffset(origToNewIdx[nodeIndex],k,pinOffset);
      }
   }

   _seenNets.clear();
   _essentialNets.clear();
   _mobileCells.clear();
}
